
using System;
using System.Collections;
using System.Text;
using Gtk;
using IPod;

namespace Dopi {

    public class PlaylistView : TreeView {

        private SongDatabase db;
        private Playlist playlist;
        private ListStore store;
        private TreeIterCompareFunc compareFunc;
        private bool syncing = false;

        public Playlist Playlist {
            get { return playlist; }
            set {
                if (playlist != null) {
                    playlist.SongAdded -= OnSongAdded;
                    playlist.SongRemoved -= OnSongRemoved;
                }
                
                playlist = value;

                if (playlist != null) {
                    playlist.SongAdded += OnSongAdded;
                    playlist.SongRemoved += OnSongRemoved;
                    Sensitive = true;
                } else {
                    Sensitive = false;
                }
            }
        }

        public SongDatabase SongDatabase {
            get { return db; }
            set { db = value; }
        }
        
        public PlaylistView () {
            this.HeadersVisible = false;
            this.Selection.Mode = SelectionMode.Multiple;
            this.Reorderable = true;
            
            store = new ListStore (typeof (Song));

            this.Model = store;

            CellRendererText cell = new CellRendererText ();
            cell.Editable = false;
            AppendColumn ("Songs", cell,
                          new TreeCellDataFunc (OnCellData));
            this.KeyPressEvent += OnKeyPress;

            this.EnableSearch = true;
            this.SearchColumn = 0;
            this.SearchEqualFunc = OnSearchEqual;

            TargetEntry[] entries = new TargetEntry[] {
                new TargetEntry ("GTK_TREE_MODEL_ROW", TargetFlags.Widget, 0),
                new TargetEntry (Utility.SongDndTarget, 0, Utility.SongDndId)
            };

            EnableModelDragSource (Gdk.ModifierType.Button1Mask, entries,
                                   Gdk.DragAction.Default | Gdk.DragAction.Move);
            EnableModelDragDest (entries, Gdk.DragAction.Default);

            this.DragDataReceived += OnDragDataReceived;
            this.DragEnd += OnDragEnd;
            
            Refresh ();
        }

        public void Refresh () {
            store.Clear ();

            if (playlist == null)
                return;

            foreach (Song song in playlist.Songs) {
                store.AppendValues (song);
            }
        }

        public void RefreshSong (Song song) {
            TreeIter iter;

            if (!store.GetIterFirst (out iter))
                return;

            do {
                Song song2 = (Song) store.GetValue (iter, 0);

                if (song2 == song) {
                    store.EmitRowChanged (store.GetPath (iter), iter);
                }
            } while (store.IterNext (ref iter));
        }

        private void OnCellData (TreeViewColumn column, CellRenderer renderer, TreeModel model,
                                 TreeIter iter) {
            Song song = (Song) store.GetValue (iter, 0);

            (renderer as CellRendererText).Text = String.Format ("{0} - {1}", song.Artist, song.Title);
        }

        private bool OnSearchEqual (TreeModel model, int column, string key, TreeIter iter) {
            Song song = (Song) store.GetValue (iter, column);

            return !song.Title.ToLower ().StartsWith (key.ToLower ());
        }

        private void RemoveSelected () {
            TreePath[] selection = this.Selection.GetSelectedRows ();

            if (selection == null || selection.Length == 0)
                return;
            
            TreeIter iter = TreeIter.Zero;
            store.GetIter (out iter, selection[0]);

            playlist.RemoveSong (selection[0].Indices[0]);

            RemoveSelected ();
        }

        private void OnKeyPress (object o, KeyPressEventArgs args) {
            if (args.Event.Key == Gdk.Key.Delete) {
                RemoveSelected ();
            }
        }

        private Song FindSongById (int id) {
            
            // blah
            foreach (Song song in db.Songs) {
                if (song.Id == id)
                    return song;
            }

            return null;
        }

        private void SyncPlaylist () {

            // sigh
            syncing = true;
            
            playlist.Clear ();

            TreeIter iter;

            if (!store.GetIterFirst (out iter))
                return;

            do {
                Song song = (Song) store.GetValue (iter, 0);
                playlist.AddSong (song);
            } while (store.IterNext (ref iter));

            syncing = false;
        }

        private void OnSongAdded (object o, int index, Song song) {
            if (syncing)
                return;
            
            TreeIter iter = store.Insert (index);
            store.SetValue (iter, 0, song);
        }

        private void OnSongRemoved (object o, int index, Song Song) {
            if (syncing)
                return;
            
            TreePath path = new TreePath ();
            path.AppendIndex (index);

            TreeIter iter = TreeIter.Zero;

            if (!store.GetIter (out iter, path))
                return;
            
            store.Remove (ref iter);
        }

        private void OnDragDataReceived (object sender, DragDataReceivedArgs args) {
            bool success = false;

            if (args.SelectionData.Target == "GTK_TREE_MODEL_ROW") {
                return;
            }
            
            if (args.SelectionData.Length >=0 && args.SelectionData.Format == 8) {
                string songIds = Encoding.ASCII.GetString (args.SelectionData.Data, 0, args.SelectionData.Length);

                ArrayList songs = new ArrayList ();
                
                foreach (string id in songIds.Trim ().Split (';')) {
                    if (id == String.Empty)
                        continue;
                    
                    Song song = FindSongById (Int32.Parse (id));

                    songs.Add (song);
                }

                TreePath path;
                TreeViewDropPosition pos;

                GetDestRowAtPos (args.X, args.Y, out path, out pos);

                int index = playlist.Songs.Length;

                if (path != null) {
                    index = path.Indices[0];

                    if (pos == TreeViewDropPosition.After ||
                        pos == TreeViewDropPosition.IntoOrAfter) {
                        index++;
                    }
                }

                foreach (Song song in songs) {
                    playlist.InsertSong (index, song);
                    index++;
                }

                SyncPlaylist ();
                
                success = true;
            }
            
            Drag.Finish (args.Context, success, false, args.Time);
        }

        private void OnDragEnd (object o, DragEndArgs args) {
            SyncPlaylist ();
        }
    }
}
